/** @format */

import { SyntheticEvent, useCallback, useRef, useState } from 'react';

export const useLongPress = ({
  onLongPress,
  onClick,
  shouldPreventDefault = true,
  delay = 750,
}: {
  onLongPress: (params?: any) => void;
  onClick: (params?: any) => void;
  shouldPreventDefault?: boolean;
  delay?: number;
}) => {
  const [longPressTriggered, setLongPressTriggered] = useState(false);
  const timeout = useRef<any>();
  const target = useRef<any>();

  const start = useCallback(
    event => {
      if (shouldPreventDefault && event.target) {
        event.target.addEventListener('touchend', preventDefault, {
          passive: false,
        });
        target.current = event.target;
      }
      timeout.current = setTimeout(() => {
        onLongPress(event);
        setLongPressTriggered(true);
      }, delay);
    },
    [onLongPress, delay, shouldPreventDefault],
  );

  const clear = useCallback(
    (event, shouldTriggerClick = true) => {
      timeout.current && clearTimeout(timeout.current);
      shouldTriggerClick && !longPressTriggered && onClick();
      setLongPressTriggered(false);
      if (shouldPreventDefault && target.current) {
        target.current.removeEventListener('touchend', preventDefault);
      }
    },
    [shouldPreventDefault, onClick, longPressTriggered],
  );

  return {
    onMouseDown: (event: SyntheticEvent) => start(event),
    onTouchStart: (event: SyntheticEvent) => start(event),
    onMouseUp: (event: SyntheticEvent) => clear(event),
    onMouseLeave: (event: SyntheticEvent) => clear(event, false),
    onTouchEnd: (event: SyntheticEvent) => clear(event),
  };
};

const isTouchEvent = (event: TouchEvent) => {
  return 'touches' in event;
};

const preventDefault = (event: TouchEvent) => {
  if (!isTouchEvent(event)) return;

  if (event.touches.length < 2 && event.preventDefault) {
    event.preventDefault();
  }
};
